//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Numeric.h	--- Auxiliary routines/constancts with numerical flavor
//
//	DESCRIPTION:
//		Collection of useful numeric routines + constants with a numerical
//		flavor. Everyhting is in namespace Numeric
//
//	CHANGE HISTORY:
//		UL, 2001-12-10: 0.1.0: - first version
//
//	AUTHOR:
//		U. Lammers, Mon Dec 10 11:37:11 MET 2001
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/Numeric.h,v 1.2 2008/09/26 16:38:55 eojero Exp $
///

#ifndef NUMERIC_H
#define NUMERIC_H

#include <cmath>

namespace Numeric {

//
//	useful numerical constants
//
#if defined(M_PI)
	#undef M_PI
#endif
const double M_PI			= 3.14159265358979323846;
const double M_HALF_PI		= .5*M_PI;
const double M_TWO_PI		= 2.*M_PI;
const double M_SQRT_2		= 1.4142135623730950488;	// sqrt(2)
const double M_SQRT_TWO_PI	= 2.5066282746310005024;	// sqrt(2*Pi)
const double M_2SQ2LN2		= 2.3548200450309493;		// 2.*sqrt(2.*ln(2))
const double EPSILON		= 1.e-12;
const double DEG_TO_RAD		= M_PI/180.;
const double RAD_TO_DEG		= 180./M_PI;


//
//	square/cube
//
template<class T> T SQR(T x) { return x*x; }
template<class T> T CUB(T x) { return x*SQR(x); }

//
//	closest integer
//
template<class RealType>
int closestInt(RealType v)
{
	return static_cast<int>(v < 0. ? std::ceil(v - .5) : std::floor(v + .5));
} // closestInt

//
//	Gamma function
//
double gammaln(double x);
	//
	//	RETURNS:
	//		Value of gamma function at <x>
	//


//
//	factorial
//
double factorial(unsigned short n);
	//
	//	RETURNS:
	//		n! - beware: only small values of n shall be accurate to
	//			 integer precision; large values of n shall result in
	//			 numerical overflows
	//

}

#endif

