//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	random_c.cc --- implementation of C I/F layer to random number generator
//
//	DESCRIPTION:
//		See Random.h/Ranlux.h/random.f90
//
//	CHANGE HISTORY:
//		UL, 2002-11-05: 1.0.0: - first version
//
//	AUTHOR:
//		U. Lammers,	Tue Nov  5 16:09:07 MET 2002
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/utils/src/random_c.cc,v 1.2 2008/09/26 16:38:55 eojero Exp $
//
#include "Ranlux.h"
#include "f90cxx.h"
#include "common.h"

extern "C" {

int CXXFUNC(sas_randrangetype, SAS_randRangeType)()
{
	return static_cast<int>(randGen.rangeType());
} // SAS_randRangeType

unsigned CXXFUNC(sas_randseed, SAS_randSeed)()
{
	return randGen.seed();
} // SAS_randSeed()

Double CXXFUNC(sas_randmin, SAS_randMin)()
{
	return randGen.min();
} // SAS_randMin()

Double CXXFUNC(sas_randmax, SAS_randMax)()
{
	return randGen.max();
} // SAS_randMax()

void CXXFUNC(sas_randsetmin, SAS_randSetMin)(const Double *min)
{
	randGen.min(*min);
} // SAS_randSetMin()

void CXXFUNC(sas_randsetmax, SAS_randSetMax)(const Double *max)
{
	randGen.max(*max);
} // SAS_randSetMax()

void CXXFUNC(sas_randsetminmaxrange, SAS_randSetMinMaxRange)
		(const Double *min, const Double *max, const Integer3 *range)
{
	randGen.minMaxRange(*min, *max,
						static_cast<RandomNumberGenerator::RangeType>(*range));
} // SAS_randSetMinMaxRange()

void CXXFUNC(sas_randpopstate, SAS_randPopState)()
{
	randGen.popState();
} // SAS_randPopState()

void CXXFUNC(sas_randreset, SAS_randReset)()
{
	randGen.reset();
} // SAS_randReset()

Double CXXFUNC(sas_rand, SAS_rand)()
{
	return randGen.rand();
} // SAS_rand()

void CXXFUNC(sas_randpushstate1, SAS_randPushState1)()
{
	randGen.pushState();
} // SAS_randPushState()

void CXXFUNC(sas_randpushstate2, SAS_randPushState2)
	(const Double *min, const Double *max, const Integer3 *rangeType)
{
	randGen.pushState(*min, *max,
				  static_cast<RandomNumberGenerator::RangeType>(*rangeType));
} // SAS_randPushState()

}
