#ifndef __SCANNER_H
#define __SCANNER_H

#include <string>
#include <sstream>
#include <istream>


#include "file.h"
#include "token.h"


class ScannerT
{
	public :

		enum StateType
		{
				NORMAL,
				FREE_FORMAT_TEXT,
				LINE
		};

				ScannerT(
					istream &	inputStream,
					char 		commentDesignator = '#',
					const string &	fileName = "NoName",
					unsigned int	lineNumber = 1,
					unsigned int	columnNumber = 1 );

				ScannerT(
					const string &	fileName,
					char 		commentDesignator = '#' );

				~ScannerT();

		//void		Accept(
		//			Token &		thisToken );

		Token		acceptNext();
		Token		Next() const;

		void		getLine(
					char *		line,
					streamsize	n );

		bool		NextIs( char			ThisCharacter ) const;
		bool		NextIs( char *			ThisString ) const;
		bool		nextIs( const string &		thisString ) const;
		bool		nextIs( Token::IDType		tokenID ) const;

		void		match(	const string &		thisString );

		void		Match(
					char *			String,
					Token &			token );

		void		Match(
					Token::IDType		tokenID,
					Token &			token );

		void		Match(
					char			ThisCharacter,
					Token &			token );

		void		match(
					char			thisCharacter );

		Token		matchOneOf(
					Token::IDType		tokenID,
					... );

		Token		MatchOneOf(
					char *			String,
					... );

		void		matchSignedNumeric(
					Token::IDType		tokenID,
					Token&			token );

		void		SetState(
					StateType		toThisState );

		StateType	StateOf() const;

		bool		EndOfFile() const;

		char		commentDesignator() const { PROC RETURN _commentDesignator; }
		StreamT &	stream() const { PROC RETURN File; }

	private :
		// Forbid
		ScannerT( const ScannerT & );
		ScannerT & operator=( const ScannerT & );

		StreamT & 	File;
		Token		LookaheadToken;
		void		FetchNextToken();
		void		FetchFreeText();
		void		SkipCommentsAndWhiteSpace();
		StateType	State;
		char		_commentDesignator;
		ScannerT *	_lineScanner;
		char *		_line;
		istringstream *	_lineStr;


};


#endif

