module amod
  use types
  implicit none

  integer, parameter, dimension(4) :: a = (/ 1,2,3,4 /)
  integer, dimension(2,3) :: b

  interface
    subroutine cxxfunction1(a)
      integer, dimension(:) :: a
    end subroutine
  end interface

  interface
    subroutine cxxfunction2(a)
      integer, dimension(:,:) :: a
    end subroutine
  end interface

  interface
    subroutine withtypes(b,s,i,f,d)
      use types
      integer(kind=int8), intent(in) :: b
      integer(kind=int16), intent(in) :: s
      integer(kind=int32), intent(in) :: i
      real(kind=single), intent(in) :: f
      real(kind=double), intent(in) :: d
    end subroutine
  end interface

contains

  subroutine printstring()
    write(*,*) "hello"
  end subroutine

  subroutine printarray1(a)
    integer, dimension(:), intent(in) :: a

    if (a(1) /= 10) call error('',"a(1) != 10")
    if (a(2) /= 20) call error('',"a(2) != 20")
    if (a(3) /= 30) call error('',"a(3) != 30")
    if (a(4) /= 40) call error('',"a(4) != 40")

  end subroutine

  subroutine printarray2(a)
    integer, dimension(:,:), intent(in) :: a
    integer :: i, j

    write(*,*) shape(a)
    write(*,*) a

    do i=lbound(a,1),ubound(a,1)
      do j=lbound(a,2),ubound(a,2)
        write(*,*) i,j,a(i,j)
      end do
    end do

    if (size(a,dim=1) /= 2) call error('',"size(a,1) != 2")
    if (size(a,dim=2) /= 3) call error('',"size(a,2) != 3")
    if (a(1,1) /= 110) call error('',"a(1,1) != 110")
    if (a(2,1) /= 120) call error('',"a(2,1) != 120")
    if (a(1,2) /= 210) call error('',"a(1,2) != 210")
    if (a(2,2) /= 220) call error('',"a(2,2) != 220")
    if (a(1,3) /= 310) call error('',"a(1,3) != 310")
    if (a(2,3) /= 320) call error('',"a(2,3) != 320")

  end subroutine

  subroutine callwitharray1()

    call cxxfunction1(a)

  end subroutine

  subroutine callwitharray2()

    b(1,:) = (/ 11,12,13 /)
    b(2,:) = (/ 21,22,23 /)
    call cxxfunction2(b)

  end subroutine

  subroutine callreplaced()
    use replace

    call replaced

  end subroutine

  subroutine printtypes(b,s,i,f,d)
    integer(kind=int8), intent(in) :: b
    integer(kind=int16), intent(in) :: s
    integer(kind=int32), intent(in) :: i
    real(kind=single), intent(in) :: f
    real(kind=double), intent(in) :: d

    if (b /= -101)       call error('',"b != -101")
    if (s /= 1001)       call error('',"s != 1000")
    if (i /= -10001)     call error('',"i != -10001")
    if (f /= 2.34e5)     call error('',"f != 2.34e5")
    if (d /= -2.3456e7)  call error('',"d != -2.3456e7")

    write(*,*) int8, int16, int32, single, double
    write(*,*) b,s,i,f,d

  end subroutine

  subroutine callwithtypes()
    integer(kind=int8)  :: b = 100
    integer(kind=int16) :: s = -1000
    integer(kind=int32) :: i = 10000
    real(kind=single)   :: f = -1.23e4
    real(kind=double)   :: d = 1.23456e7

    call alltypes(b,s,i,f,d)

  end subroutine
   

end module amod
